#ifndef RESOURCES_H
#define RESOURCES_H
#include "../server/resource.h"

namespace http::auth {
    /**
    * Класс пользователя, содержит логин/хеш_пароля/настройки пользователя/права.
    * Хеш пароля представляется в виде строки
    */
    class User {
    private:
        uint32_t perms;

    public:
        const std::string username;
        std::string passwordHash;

        User(const std::string& username, const std::string& passwordHash);

        /**
         * Проверить пароль на соответствие хешу
         * @param pass
         * @return
         */
        bool checkPassword(const std::string& pass);

        /**
         * Установка пароля
         * @param pass строка с исходным паролем
         */
        void setPassword(const std::string& pass);


        static constexpr uint32_t SUPERUSER = 0x0001;
        static constexpr uint32_t WATCH_STATISTICS = 0x0002; // мониторинг модема
        static constexpr uint32_t RESET_PACKET_STATISTICS = 0x0004; // сброс статистики пакетов
        static constexpr uint32_t WATCH_SETTINGS = 0x0008; // просмотр настроек , если недоступно, то вкладки с настройками не будет
        static constexpr uint32_t EDIT_SETTINGS = 0x0010; // редактирование настроек, установка параметров модулятора/демодулятора/dma/cinc
        static constexpr uint32_t UPDATE_FIRMWARE = 0x0020; // обновление прошивки

        /**
         * Проверить, что у пользователя есть нужное право. Если это суперпользователь, то у него по умолчанию все права есть.
         * @param p набор прав, из констант данного класса.
         * @return
         */
        bool checkPremisions(uint32_t p);

        void setPremisions(uint32_t p);
        void resetPremisions(uint32_t p);

        ~User();
    };

    /**
    * Класс аутентификации. Управляет всеми сессиями, создает новые при логине, удаляет при логауте.
    * @note Класс устанавливает заголовок 'Set-Cookie' в ответе, и этот заголовок должен дойти до пользователя!
    */
    class AuthProvider {
    public:
        AuthProvider();

        /**
         * Авторизовать пользователя.
         *
         * @param rep
         * @return true, в случае успешной авторизации
         */
        bool doAuth(const std::string& username, const std::string& password, server::Reply& rep);

        ~AuthProvider();
    };

    class NeedAuentificationResource : public resource::BasicResource {
    };
}

#endif //RESOURCES_H
